@@ -351,7 +351,9 @@ private:
 	/// whether the object was contructed with duplicated = true
 	bool duplicated;
 
+#ifdef	CCXX_PACKED
 #pragma pack(1)
+#endif
 	/**
 	 * @struct RTPFixedHeader
 	 * @short RTP fixed header as it is send through the network.
@@ -394,6 +396,7 @@ private:
 	 * use this by recasing the pointer returned by getPayload().
 	 */
 
+public:
 	struct RFC2833Payload
 	{
 #if __BYTE_ORDER == __BIG_ENDIAN
@@ -410,7 +413,8 @@ private:
         	uint16 duration : 16;
 #endif
 	};	
-	
+
+private:	
 	/**
 	 * @struct RTPHeaderExt
 	 *
@@ -423,10 +427,13 @@ private:
 		uint16 undefined; ///< to be defined 
 		uint16 length;    ///< number of 32-bit words in the extension
 	};
+#ifdef	CCXX_PACKED
 #pragma pack()
+#endif
 
 	/* definitions for access to most common 2833 fields... */
 
+public:
 	/**
 	 * Fetch a raw 2833 packet.
 	 *
