@@ -14,34 +14,32 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of ccRTP.
-// 
-// The exception is that, if you link the ccRTP library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccRTP library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccRTP.  If you copy code from other releases into a copy of
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
+//
+// This exception applies only to the code released under the name GNU
+// ccRTP.  If you copy code from other releases into a copy of GNU
 // ccRTP, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccRTP, it is your choice
+//
+// If you write modifications of your own for GNU ccRTP, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
-#include <algorithm>
 #include "private.h"
 #include <ccrtp/pool.h>
 
+#include <algorithm>
+
 #ifdef  CCXX_NAMESPACES
 namespace ost {
 using std::list;
@@ -150,8 +148,86 @@ SingleRTPSessionPool::run()
 #endif // ndef WIN32
 }
 
+#if defined(_MSC_VER) && _MSC_VER >= 1300
+SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>(
+		const InetHostAddress& ia, 
+			       tpport_t dataPort, 
+			       tpport_t controlPort,
+			       int pri,
+			       uint32 memberssize,
+			       RTPApplication& app):
+		Thread(pri),
+		TRTPSessionBase<RTPDataChannel,RTCPChannel,ServiceQueue>
+	(ia,dataPort,controlPort,memberssize,app)
+	{ }
+
+SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>(
+	const InetMcastAddress& ia,
+			       tpport_t dataPort, 
+			       tpport_t controlPort, 
+			       int pri,
+			       uint32 memberssize,
+			       RTPApplication& app,
+			       uint32 iface): 
+		Thread(pri),
+		TRTPSessionBase<RTPDataChannel,RTCPChannel,ServiceQueue>
+	(ia,dataPort,controlPort,memberssize,app,iface)
+	{ }
+
+void SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::startRunning()
+{
+	enableStack();
+	Thread::start();
+}
+
+bool SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::isPendingData(microtimeout_t timeout)
+{
+	return TRTPSessionBase<RTPDataChannel,RTCPChannel,ServiceQueue>::isPendingData(timeout);
+}
+
+void SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::timerTick(void)
+{
+}
+
+void SingleThreadRTPSession<DualRTPUDPIPv4Channel,DualRTPUDPIPv4Channel,AVPQueue>::run(void)
+{
+		microtimeout_t timeout = 0;
+		while ( ServiceQueue::isActive() ) {
+			if ( timeout < 1000 ){ // !(timeout/1000)
+				timeout = getSchedulingTimeout();
+			}
+			setCancel(cancelDeferred);
+			controlReceptionService();
+			controlTransmissionService();
+			setCancel(cancelImmediate);
+			microtimeout_t maxWait = 
+				timeval2microtimeout(getRTCPCheckInterval());
+			// make sure the scheduling timeout is
+			// <= the check interval for RTCP
+			// packets
+			timeout = (timeout > maxWait)? maxWait : timeout;
+			if ( timeout < 1000 ) { // !(timeout/1000)
+				setCancel(cancelDeferred);
+				dispatchDataPacket();
+				setCancel(cancelImmediate);
+				timerTick();
+			} else {
+				if ( isPendingData(timeout/1000) ) {
+					setCancel(cancelDeferred);
+					takeInDataPacket();
+					setCancel(cancelImmediate);
+				}
+				timeout = 0;
+			}
+		}
+		dispatchBYE("GNU ccRTP stack finishing.");
+		sleep(~0);
+}
+
+#endif
+
 #ifdef  CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
